@@ -45,6 +45,7 @@
 #define ADD_LIST_DELAY 30
 #define ADD_LIST_CHUNK 400
 #define SCROLL_DELAY   20
+#define DEF_THUMB_SIZE 128
 
 enum {
 	BUSY,
@@ -82,6 +83,9 @@ load_thumb_error_cb (ThumbLoader *tl,
 	GthFileList *file_list = data;
 	float        percent;
 
+	if (file_list == NULL)
+		return;
+
 	gth_file_view_set_unknown_pixbuf (file_list->view,  file_list->thumb_pos);
 
 	file_list->thumb_fd->error = TRUE;
@@ -104,6 +108,9 @@ load_thumb_done_cb (ThumbLoader *tl,
 	GthFileList *file_list = data;
 	float        percent;
 
+	if (file_list == NULL)
+		return;
+
 	update_thumb_in_clist (file_list);
 	file_list->thumb_fd->error = FALSE;
 	file_list->thumb_fd->thumb = TRUE;
@@ -135,6 +142,11 @@ gth_file_list_finalize (GObject *object)
 
         g_return_if_fail (GTH_IS_FILE_LIST (object));
 	file_list = GTH_FILE_LIST (object);
+	
+	if (file_list->thumb_fd != NULL) {
+		file_data_unref (file_list->thumb_fd);
+		file_list->thumb_fd = NULL;
+	}
 
 	gth_file_list_free_list (file_list);
 	g_object_unref (file_list->thumb_loader);
@@ -237,9 +249,9 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->list              = NULL;
 	file_list->sort_method       = pref_get_arrange_type ();
 	file_list->sort_type         = pref_get_sort_order ();
-	file_list->show_dot_files    = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES);
-	file_list->enable_thumbs     = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS);
-	file_list->thumb_size        = eel_gconf_get_integer (PREF_THUMBNAIL_SIZE);
+	file_list->show_dot_files    = eel_gconf_get_boolean (PREF_SHOW_HIDDEN_FILES, FALSE);
+	file_list->enable_thumbs     = eel_gconf_get_boolean (PREF_SHOW_THUMBNAILS, TRUE);
+	file_list->thumb_size        = eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, DEF_THUMB_SIZE);
 	file_list->doing_thumbs      = FALSE;
 	file_list->thumb_loader      = THUMB_LOADER (thumb_loader_new (NULL, file_list->thumb_size, file_list->thumb_size));
 	file_list->thumbs_num        = 0;
@@ -254,11 +266,11 @@ gth_file_list_init (GthFileList *file_list)
 	file_list->starting_update   = FALSE;
 
 	g_signal_connect (G_OBJECT (file_list->thumb_loader), 
-			  "done",
+			  "thumb_done",
 			  G_CALLBACK (load_thumb_done_cb), 
 			  file_list);
 	g_signal_connect (G_OBJECT (file_list->thumb_loader), 
-			  "error",
+			  "thumb_error",
 			  G_CALLBACK (load_thumb_error_cb), 
 			  file_list);
 	
@@ -267,10 +279,10 @@ gth_file_list_init (GthFileList *file_list)
 	/* image list. */
 
 	if (pref_get_view_as () == GTH_VIEW_AS_THUMBNAILS)
-		file_list->view = gth_file_view_thumbs_new (eel_gconf_get_integer (PREF_THUMBNAIL_SIZE) + THUMB_BORDER);
+		file_list->view = gth_file_view_thumbs_new (eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, DEF_THUMB_SIZE) + THUMB_BORDER);
 
 	else if (pref_get_view_as () == GTH_VIEW_AS_LIST)
-		file_list->view = gth_file_view_list_new (eel_gconf_get_integer (PREF_THUMBNAIL_SIZE) + THUMB_BORDER);
+		file_list->view = gth_file_view_list_new (eel_gconf_get_integer (PREF_THUMBNAIL_SIZE, DEF_THUMB_SIZE) + THUMB_BORDER);
 
 	gth_file_view_enable_thumbs (file_list->view, file_list->enable_thumbs);
 
@@ -449,7 +461,7 @@ set_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 }
 
 
-void
+static void
 set_list__step2 (GetFileInfoData *gfi_data)
 {
 	GnomeVFSAsyncHandle *handle;
@@ -502,7 +514,7 @@ gth_file_list_set_list (GthFileList *file_list,
 					   done_func, 
 					   done_func_data);
 
-	fast_file_type = eel_gconf_get_boolean (PREF_FAST_FILE_TYPE);
+	fast_file_type = eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE);
 
 	for (scan = new_list; scan; scan = scan->next) {
 		char        *full_path = scan->data;
@@ -694,7 +706,7 @@ add_list__get_file_info_done_cb (GnomeVFSAsyncHandle *handle,
 }
 
 
-void
+static void
 add_list__step2 (GetFileInfoData *gfi_data)
 {
 	GnomeVFSAsyncHandle *handle;
@@ -748,7 +760,7 @@ gth_file_list_add_list (GthFileList *file_list,
 
 		if ((! gfi_data->file_list->show_dot_files 
 		     && file_is_hidden (name_only))
-		    || ! file_is_image (full_path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE)))
+		    || ! file_is_image (full_path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE)))
 			continue;
 		
 		escaped = gnome_vfs_escape_path_string (full_path);
@@ -819,7 +831,7 @@ it_data_free (InterruptThumbsData *it_data)
 /* -- -- */
 
 
-void
+static void
 set_sort_method__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList    *file_list = it_data->file_list;
@@ -866,7 +878,7 @@ gth_file_list_set_sort_method (GthFileList   *file_list,
 /* -- -- */
 
 
-void
+static void
 set_sort_type__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList *file_list = it_data->file_list;
@@ -1227,7 +1239,7 @@ gth_file_list_prev_image (GthFileList *file_list,
 /* -- -- */
 
 
-void
+static void
 delete_pos__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList *file_list = it_data->file_list;
@@ -1280,7 +1292,7 @@ gth_file_list_delete_pos (GthFileList *file_list,
 /* -- -- */
 
 
-void
+static void
 rename_pos__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList    *file_list = it_data->file_list;
@@ -1336,7 +1348,7 @@ gth_file_list_rename_pos (GthFileList *file_list,
 /* -- -- */
 
 
-void
+static void
 update_comment__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList   *file_list = it_data->file_list;
@@ -1386,7 +1398,7 @@ gth_file_list_update_comment (GthFileList *file_list,
 /* -- -- */
 
 
-void
+static void
 set_thumbs_size__step2 (InterruptThumbsData *it_data)
 {
 	GthFileList *file_list = it_data->file_list;
@@ -1397,23 +1409,16 @@ set_thumbs_size__step2 (InterruptThumbsData *it_data)
 	g_object_unref (G_OBJECT (file_list->thumb_loader));
 	file_list->thumb_loader = THUMB_LOADER (thumb_loader_new (NULL, size, size));
 	g_signal_connect (G_OBJECT (file_list->thumb_loader), 
-			  "done",
+			  "thumb_done",
 			  G_CALLBACK (load_thumb_done_cb), 
 			  file_list);
 	g_signal_connect (G_OBJECT (file_list->thumb_loader), 
-			  "error",
+			  "thumb_error",
 			  G_CALLBACK (load_thumb_error_cb), 
 			  file_list);
 
 	gth_file_view_set_image_width (file_list->view, size + THUMB_BORDER);
 
-	/* FIXME
-	if (it_data->restart_thumbs) {
-		file_list->doing_thumbs = TRUE;
-		gth_file_list_update_next_thumb (file_list);
-	}
-	*/
-
 	it_data_free (it_data);
 }
 
@@ -1510,8 +1515,9 @@ gth_file_list_update_thumb (GthFileList  *file_list,
 	fd->thumb = FALSE;
 
 	file_list->thumb_pos = pos;
+	if (file_list->thumb_fd != NULL)
+		file_data_unref (file_list->thumb_fd);
 	file_list->thumb_fd = fd;
-	file_data_unref (fd);
 
 	gth_file_list_update_current_thumb (file_list);
 }
@@ -1558,6 +1564,11 @@ gth_file_list_thumb_cleanup (GthFileList *file_list)
 	file_list->thumbs_num = 0;
 	file_list->doing_thumbs = FALSE;
 
+	if (file_list->thumb_fd != NULL) {
+		file_data_unref (file_list->thumb_fd);
+		file_list->thumb_fd = NULL;
+	}
+
 	if (file_list->progress_func) 
 		file_list->progress_func (0.0, file_list->progress_data);
 }
@@ -1612,7 +1623,11 @@ gth_file_list_update_next_thumb (GthFileList *file_list)
 
 	file_list->thumb_pos = new_pos;
 	file_list->thumbs_num++;
+
+	if (file_list->thumb_fd != NULL)
+		file_data_unref (file_list->thumb_fd);
 	file_list->thumb_fd = fd;
+	file_data_ref (file_list->thumb_fd);
 
 	gth_file_list_update_current_thumb (file_list);
 }
@@ -1626,7 +1641,7 @@ gth_file_list_update_thumbs (GthFileList *file_list)
 	if (! file_list->enable_thumbs)
 		return;
 
-	thumb_loader_set_max_file_size (THUMB_LOADER (file_list->thumb_loader), eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT));
+	thumb_loader_set_max_file_size (THUMB_LOADER (file_list->thumb_loader), eel_gconf_get_integer (PREF_THUMBNAIL_LIMIT, 0));
 
 	for (scan = file_list->list; scan; scan = scan->next) {
 		FileData *fd = scan->data;
